#!/usr/bin/env python

import gobject
import pygtk
pygtk.require("2.0")
import gtk
import logging
logger = logging.getLogger("PyGTK UI")
from threading import Thread,Lock
from songanalysiscommon import *


def _ppiholder(): pass
def get_plugin_public_interface(): return _ppiholder.pi
def _set_plugin_public_interface(pi): _ppiholder.pi = pi

get_ui_config_lock = Lock()

def get_ui_config():
	global get_ui_config_lock
	get_ui_config_lock.acquire()
	try:
		c = get_plugin_public_interface().get_config()
		def has_key(object,key): return object.has_key(key)
		if not has_key(c,"ui"):
			c["ui"] = dict()
		if not has_key(c["ui"],"SearchWindow"):
			c["ui"]["SearchWindow"] = dict()
		if not has_key(c["ui"]["SearchWindow"],"height"):
			c["ui"]["SearchWindow"]["height"] = 350
		if not has_key(c["ui"]["SearchWindow"],"width"):
			c["ui"]["SearchWindow"]["width"] = 600
		if not has_key(c["ui"]["SearchWindow"],"columns"):
			c["ui"]["SearchWindow"]["columns"] = [ ["File name",1,True,False,120],["Folder",2,True,False,300],["Artist",3,True,True,110],["Album",4,True,True,110],["Title",5,True,True,110],["Similarity",6,False,True],["Tempo",7,False,True] ]
		if not has_key(c["ui"]["SearchWindow"],"num_results"):
			c["ui"]["SearchWindow"]["num_results"] = 10
		if not has_key(c["ui"]["SearchWindow"],"weighting"):
			c["ui"]["SearchWindow"]["weighting"] = 1.0
	
		return c["ui"]
	finally:
		get_ui_config_lock.release()


# extracted from amarok-songanalysis, should be librarized
class HIGErrorAlert(gtk.Dialog):
	def __init__(self,parent,title,summary,message):
		flags = gtk.DIALOG_NO_SEPARATOR
		if parent: flags = flags | gtk.DIALOG_MODAL | gtk.DESTROY_WITH_PARENT
		gtk.Dialog.__init__(self,title,parent,flags=flags)
		self.set_resizable(False)
		self.add_button(gtk.STOCK_CLOSE,gtk.RESPONSE_CLOSE)
		self.connect_object("response",self.response,self)
		wsummary = gtk.Label()
		wsummary.set_markup("<b><big>%s</big></b>"%summary)

		wmessage = gtk.Label()
		wmessage.set_line_wrap(True)
		wmessage.set_markup(message)

		for a in [wsummary,wmessage]:
			a.set_alignment(0.5,0.5)
			a.show()

		icon = gtk.Image()
		icon.set_from_stock(gtk.STOCK_DIALOG_ERROR,gtk.ICON_SIZE_DIALOG)
		icon.show()

		hbox = gtk.HBox()
		hbox.set_spacing(12)
		hbox.show()


		hbox.pack_start(icon,expand=False,fill=False)

		vbox = gtk.VBox()
		vbox.set_spacing(12)
		vbox.show()
		
		vbox.pack_start(wsummary,expand=False,fill=False)
		vbox.pack_start(wmessage,expand=False,fill=False)

		hbox.pack_start(vbox,expand=False,fill=False)

		self.vbox.pack_start(hbox,expand=False,fill=False)
		self.vbox.set_spacing(12)
		self.vbox.set_border_width(0)

		self.set_border_width(12)
		self.action_area.set_border_width(0)

	
	def response(self,response=None,extra=None):
		self.destroy()









# private implementation

class SearchParameters(gtk.Table):
	def __init__(self):
		
		gtk.Table.__init__(self,rows=3,columns=3)
# 		self.set_expanded(True)
		
# 		align = gtk.Alignment(xscale=1,yscale=1)
# 		align.set_padding(6,0,12,0)
		
		
		
		
		self.set_border_width(0)
		self.set_row_spacings(12)
		self.set_col_spacings(12)
		
		
		image = gtk.image_new_from_stock(gtk.STOCK_FIND,gtk.ICON_SIZE_DIALOG)
		self.attach(image,0,1,0,2,xoptions=0,yoptions=0)
		
		label = gtk.Label("Results:")
		label.set_alignment(0.0,0.0)
		self.attach(label,1,2,0,1,xoptions=gtk.FILL,yoptions=0)
		
		adj1 = gtk.Adjustment(value=get_ui_config()["SearchWindow"]["num_results"],lower=5,upper=1000,step_incr=1,page_incr=10,page_size=0)
		self.number_of_songs = gtk.SpinButton(adj1)
		self.attach(self.number_of_songs,2,3,0,1)
		
		label2 = gtk.Label("Songs similar in:")
		label2.set_alignment(0.0,0.0)
		self.attach(label2,1,2,1,2,xoptions=gtk.FILL,yoptions=0)
		
		label3 = gtk.Label("Tempo")
		adj2 = gtk.Adjustment(value=get_ui_config()["SearchWindow"]["weighting"],lower=0.0,upper=2.0,step_incr=0.1,page_incr=0.5,page_size=0)
		self.weighting = gtk.HScale(adj2)
		self.weighting.set_size_request(100,-1)
		self.weighting.set_range(0.0,2.0)
		self.weighting.set_draw_value(False)
		label4 = gtk.Label("Spectrum")
		
		hbox = gtk.HBox(spacing=8)
		for a in (label3,self.weighting,label4):
			hbox.pack_start(a,expand=False,fill=False)
			hbox.set_child_packing(self.weighting,True,True,0,gtk.PACK_START)
		self.attach(hbox,2,3,1,2)
		
		self.connect("destroy",self.on_destroy)
		
	def on_destroy(self,w,e=None):
		get_ui_config()["SearchWindow"]["weighting"] = self.weighting.get_value()
		get_ui_config()["SearchWindow"]["num_results"] = self.number_of_songs.get_value()
		
# 		align.add(self.table)
# 		self.add(align)
		
		
class ResultsListView(gtk.TreeView):
	
	def __init__(self):
		gtk.TreeView.__init__(self)
		self.columns = get_ui_config()["SearchWindow"]["columns"]
		self.set_size_request(300,100)
		model = gtk.ListStore(str,str,str,str,str,str,float,float) # url, filename, folder, artist, album, title, compat, bpm
		self.set_model(model)
		self.set_rules_hint(True)
		self.set_enable_search(True)
		self.set_search_column(1)
		renderer = gtk.CellRendererText()
# 		try:
		logger.debug("Applying column info: %s"%self.columns)
		for cspec in self.columns:
			c = self.insert_column_with_attributes(-1,cspec[0],renderer,text=cspec[1])
			c.set_resizable(cspec[2])
			if cspec[2]:
				c.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
				c.set_min_width(80)
				if len(cspec) > 4: c.set_fixed_width(cspec[4])
			c.set_sort_indicator(True)
			c.set_sort_column_id(cspec[1])
			c.set_visible(cspec[3])
# 		except: logger.exception("Failed!")
		
		TARGETS = [
			('text/uri-list', 0, 0),
# 			('text/plain', 0, 0),
# 			('TEXT', 0, 0),
# 			('STRING', 0, 0),
			]

		self.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, TARGETS, gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)
		
		self.connect("drag-data-get", self.handle_drag_drop)
		self.connect("destroy", self.on_destroy)
		
	def on_destroy(self,w,e=None):
# 		logger.debug("The results list view is being destroyed")
		for a in range(len(self.columns)):
# 			resizable = self.get_column(a).get_resizable()
# 			visible = self.get_column(a).get_visible()
			fixed_width = (self.get_column(a).get_sizing() == gtk.TREE_VIEW_COLUMN_FIXED)
			if fixed_width:
				width = self.get_column(a).get_width()
				if width:
					try: self.columns[a][4] = width
					except IndexError: self.columns[a].append(width)
		logger.debug("Saved column info: %s"%self.columns)
# 			logger.debug("Columns: %s",ResultsListView.columns)
		
		
# 		self.connect("row-activated", self.handle_double_click)

# 	def handle_double_click(self,widget,extra=None,extra2=None):
# # 		print "activated"
# 		treeview = self.get_widget("matching_songs")
# 		treeselection = treeview.get_selection()
# 		model, iter = treeselection.get_selected()
# 		path = model.get_value(iter, 0)
# 		self.get_widget("chosen_song").set_filename(path)
	def handle_drag_drop(self,treeview, context, selection, info, timestamp):
# 		print "something dragged"
		treeselection = treeview.get_selection()
		model, iter = treeselection.get_selected()

		text = model.get_value(iter, 0)
		# let's recode shit
# 		text = unicode(text,"utf-8")
# 		text = text.encode("iso-8859-1")
		import urllib
		text = "file://" + urllib.pathname2url(text)
# 		print selection.target
		selection.set(selection.target, 8, text)

class SearchWindowPane(gtk.VBox):
	def __init__(self):
		
		gtk.VBox.__init__(self,spacing=12)
		scrolled_window = gtk.ScrolledWindow()
		scrolled_window.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
		scrolled_window.set_shadow_type(gtk.SHADOW_IN)
		vbox = gtk.VBox(spacing=2)
		mylabel = gtk.Label("Search results:")
		mylabel.set_alignment(0,0)
		
		hbox = gtk.HBox(spacing=12)
		
		self.search_parameters = SearchParameters()
		hbox.pack_start(self.search_parameters,expand=True,fill=True)

		buttonbox = gtk.VButtonBox()
		buttonbox.set_spacing(12)
		buttonbox.set_layout(gtk.BUTTONBOX_START)
		self.research_button = gtk.Button(stock=gtk.STOCK_FIND)
		self.stop_button = gtk.Button(stock=gtk.STOCK_STOP)
		self.stop_button.set_sensitive(False)
		buttonbox.pack_start(self.research_button)
		buttonbox.pack_start(self.stop_button)
		hbox.pack_start(buttonbox,expand=False,fill=False)
		
		self.pack_start(hbox,expand=False,fill=False)
		
		self.results_list_view = ResultsListView()
		scrolled_window.add(self.results_list_view)
		vbox.pack_start(mylabel,expand=False,fill=False)
		vbox.pack_start(scrolled_window)
		self.pack_start(vbox)
		
		buttonbox = gtk.HButtonBox()
		buttonbox.set_spacing(12)
		buttonbox.set_layout(gtk.BUTTONBOX_END)
		
		self.close_button = gtk.Button(stock=gtk.STOCK_CLOSE)
		buttonbox.pack_start(self.close_button)
		self.pack_start(buttonbox,expand=False,fill=False)
		
# 		alignment = gtk.Alignment()
# 		alignment.add(self.research_button)
# 		alignment.set(1,0.5,0,0)


import os
from os.path import basename,dirname
class SearchWindow(gtk.Window):
		
	stopped = False
	
	def __init__(self,start_file):
		gtk.Window.__init__(self)
		
		logger.info("Searching (sync) for %s"%start_file)
		
		self.start_file = start_file
		self.pane = SearchWindowPane()
# 		(str,str,str,float,float)
		self.model = self.pane.results_list_view.get_model()
		self.add(self.pane)
		self.set_border_width(12)
		self.destroyed = False
		
		self.pane.research_button.connect_object("clicked",SearchWindow.search,self)
		self.pane.stop_button.connect_object("clicked",SearchWindow.stop_search,self)
		self.pane.close_button.connect_object("clicked",SearchWindow.destroy,self)
		self.connect("delete_event",SearchWindow.destroy)
		song = get_plugin_public_interface().get_song(start_file)
		self.set_title("Songs similar to %s - %s - amaroK Smart DJ search"%(song.artist,song.title))
		self.set_default_size(get_ui_config()["SearchWindow"]["width"],get_ui_config()["SearchWindow"]["height"])
		self.pane.results_list_view.realize()
		self.pane.results_list_view.columns_autosize()
		self.pane.show_all()
		
	def destroy(self,w=None,e=None):
		logger.debug("We (SearchWindow) are being destroyed")
		get_ui_config()["SearchWindow"]["width"] = self.get_size()[0]
		get_ui_config()["SearchWindow"]["height"] = self.get_size()[1]
		self.destroyed = True
		gtk.Window.destroy(self)
		return True
		
	def stop_search(self,w=None,e=None):
		self.stopped = True
	
	def search(self,w=None,e=None):
		
		self.stopped = False
		self.model.clear()
			
		self.pane.research_button.set_sensitive(False)
		self.pane.stop_button.set_sensitive(True)
		self.pane.search_parameters.set_sensitive(False)
		weighting = self.pane.search_parameters.weighting.get_value()
		limit = int(self.pane.search_parameters.number_of_songs.get_value())
		start_file = self.start_file
		
# 		si slider hacia tempo, value = 0
# 		si slider hacia spectrum value = 2
# 		si slider en la mitad value = 1
		
		tempo_weight = 2 - weighting
		spectrum_weight = weighting
		
		logger.debug("Searching for %s max %s results with tempo/spectrum %s/%s",start_file,limit,tempo_weight,spectrum_weight)
		
		results = get_plugin_public_interface().get_similar_songs(start_file,spectrum_weight,tempo_weight,limit)
		
		def fill_one(model,r,artist,album,title):
			try:
				if self.flags() & gtk.IN_DESTRUCTION or self.destroyed:
					logger.debug("Will not set meta info because we are being destroyed")
					return
				model.append(   (r[0],basename(r[0]),dirname(r[0]),artist,album,title,r[1],r[2])    )
			except AmarokDcopFailed,e:
				if e.output == "output call failed":
					logger.debug("amaroK is unexpectedly gone")
				else: raise
			except:
					logger.exception("fill_one failed")
					
		def fill_last():
			try:
				if self.flags() & gtk.IN_DESTRUCTION or self.destroyed:
					logger.debug("Will not fill_last because we are being destroyed")
					return
				self.pane.research_button.set_sensitive(True)
				self.pane.search_parameters.set_sensitive(True)
				self.pane.stop_button.set_sensitive(False)
			except:
					logger.exception("fill_last failed")
		
		def fill_the_rest_async(model,results):
			try:
				for r in results:
					if self.stopped: break
					if self.flags() & gtk.IN_DESTRUCTION or self.destroyed:
						logger.debug("Will stop filling because we are being destroyed")
						return
					get_song = get_plugin_public_interface().get_song
					song_object = get_song(r[0])
					artist = song_object.artist
					album = song_object.album
					title = song_object.title
					gobject.idle_add(fill_one,model,r,artist,album,title)
				gobject.idle_add(fill_last)
				logger.debug("Searching for %s is done", self.start_file)
			except AmarokDcopFailed,e:
				if e.output == "output call failed":
					logger.debug("amaroK is unexpectedly gone")
				else: raise
			except:
					logger.exception("fill_the_rest_async failed")
				
		t = Thread(target=fill_the_rest_async,args=(self.model,results))
		t.setDaemon(True)
		t.start()
			
		self.pane.results_list_view.grab_focus()
	
	
def _search_sync(start_files):
	try:
		for f in start_files:
			s = SearchWindow(f)
			try:
				s.search()
				s.present()
			except NotAnalyzedYet,e:
				s.destroy()
				HIGErrorAlert(None,"Cannot search for similar songs","The selected song (%s) has not been scanned yet","The selected song has not yet been analyzed, and thus it cannot be compared to other songs in your collection.  Please right-click the song, select Smart DJ->Analyze selection now, and try again in 30 seconds."%f).show()
			except UnAnalyzable,e:
				s.destroy()
				HIGErrorAlert(None,"Cannot search for similar songs","The selected song was not scanned","The selected song (%s) cannot be analyzed, probably because its compression format was not understood by songanalysis."%f).show()
	except AmarokDcopFailed,e:
		if e.output == "output call failed":
			logger.debug("amaroK is unexpectedly gone")
		else: raise
	except:
		logger.exception("Something went wrong in _search_sync")
		HIGErrorAlert(None,"Error while searching for similar songs","Cannot search for similar songs","An unknown error just occurred.  Please report the contents of the file %s to the Smart DJ plug-in developers."%os.path.expanduser(n_logfile)).show()

class ConfigWindowDynamicModeFrame(gtk.Frame):
	
	def __init__(self):
		
		gtk.Frame.__init__(self,"")
		self.set_shadow_type(gtk.SHADOW_NONE)
		self.get_label_widget().set_markup("<b>Auto DJ mode</b>")
		
		self._super_dynamic_mode = gtk.CheckButton("Automatically add similar songs to the end of the playlist")
		
		adj = gtk.Adjustment(value=3,lower=1,upper=20,step_incr=1,page_incr=5,page_size=0)
		self._upcoming_songs = gtk.SpinButton(adj)
		
		adj = gtk.Adjustment(value=5,lower=1,upper=20,step_incr=1,page_incr=5,page_size=0)
		self._top_number_of_songs = gtk.SpinButton(adj) # top_number_of_songs
		
		adj = gtk.Adjustment(value=1.0,lower=0.0,upper=2.0,step_incr=0.1,page_incr=0.5,page_size=0)
		self._weighting = gtk.HScale(adj) # weighting
		self._weighting.set_range(0.0,2.0)
		self._weighting.set_draw_value(False)
		self._weighting.set_size_request(100,-1)
		
		self._the_song_im_listening = gtk.RadioButton(None,"The song I'm currently listening")
		
		self._the_song_im_listening_wander = gtk.RadioButton(None,"The song I'm currently listening\n(and subsequently added songs based on the latest automatically added song)")
# 		self._the_song_im_listening_wander.set_sensitive(False)
		
		self._the_last_song_in_playlist = gtk.RadioButton(None,"The last song in the playlist")
# 		self._the_last_song_in_playlist.set_sensitive(False)
		
		self._the_song_im_listening.set_group(self._the_last_song_in_playlist)
		self._the_song_im_listening_wander.set_group(self._the_last_song_in_playlist)
		
# 		self._subsequent_the_song_im_listening = gtk.RadioButton(None,"Add subsequent songs based on the song I'm listening")
		
		
		
		self._after_the_song_im_listening = gtk.RadioButton(None,"after the song I'm listening")
		self._after_the_song_im_listening.set_sensitive(False)
		
		self._at_the_end_of_playlist = gtk.RadioButton(None,"at the end of the playlist")
		
		self._after_the_song_im_listening.set_group(self._at_the_end_of_playlist)
		
		self._apply_button = gtk.Button(stock=gtk.STOCK_APPLY)
		self._apply_button.get_child().get_child().get_children()[1].set_text("Repopulate")
		
		self._settings_table = gtk.VBox(spacing=8)
		
		cell0 =gtk.HBox(spacing=6)
		
		cell05 =gtk.HBox(spacing=6)
		cell05.pack_start(gtk.Label("By selecting one of the top"),expand=False,fill=False)
		cell05.pack_start(self._top_number_of_songs,expand=False,fill=False)
		cell05.pack_start(gtk.Label("similar songs"),expand=False,fill=False)
		cell05.pack_start(gtk.Label("in Tempo"),expand=False,fill=False)
		cell05.pack_start(self._weighting,expand=True,fill=True)
		cell05.pack_start(gtk.Label("Spectrum"),expand=False,fill=False)

		cell1 = gtk.HBox(spacing=6)
		for w in [ gtk.Label("Add to the playlist at least"),self._upcoming_songs,gtk.Label("songs after the current one, based upon:") ]:
			cell1.pack_start(w,expand=False,fill=False)
		
		cell2 = self._the_last_song_in_playlist
		cell25 = self._the_song_im_listening
		cell3 = self._the_song_im_listening_wander
		
		for a in [ cell1,cell2,cell25,cell3,cell05 ]:
			self._settings_table.pack_start(a,expand=False,fill=False)
		
# 		cell00 = gtk.Label("Choose from:")
# 		cell00.set_alignment(0.0,0.0)
# 		cell10 = gtk.Label("Based on:")
# 		cell10.set_alignment(0.0,0.0)
# 		cell30 = gtk.Label("Upcoming songs:")
# 		cell30.set_alignment(0.0,0.0)
# 		cell20 = gtk.Label("Insert songs:")
# 		cell20.set_alignment(0.0,0.0)


# 		cell101 = gtk.HBox(spacing=6)
		
# 		cell101.pack_start(gtk.Label("Add at least"),expand=False,fill=False)
# 		cell101.pack_start(upcoming_songs,expand=False,fill=False)
# 		cell101.pack_start(gtk.Label("songs ahead what's playing"),expand=False,fill=False)

# 		align = gtk.Alignment(xalign=1,xscale=0,yscale=0)
# 		align.add(apply_button)
# 		cell101.pack_start(align,expand=True,fill=True)

		
# 		cell11 = gtk.HBox(spacing=12)
# 		cell11.pack_start(the_last_song_in_playlist,expand=False,fill=False)
# 		cell11.pack_start(the_song_im_listening,expand=False,fill=False)
		
# 		cell21 = gtk.HBox(spacing=12)
# # 		at_the_end_of_playlist.set_group(after_the_song_im_listening)
# 		cell21.pack_start(at_the_end_of_playlist,expand=False,fill=False)
# 		cell21.pack_start(after_the_song_im_listening,expand=False,fill=False)
		
# 		array = [ [ cell00 , cell01 ] ] #, [ cell10 , cell11 ] , [ cell20 , cell21 ] , [ cell30 , cell31 ] ]
# 		table = gtk.Table(rows=4,columns=2)
# 		table.set_border_width(0)
# 		table.set_row_spacings(8)
# 		table.set_col_spacings(12)
		
# 		table.attach(super_dynamic_mode,0,2,0,1,xoptions=gtk.FILL|gtk.EXPAND)
		
# 		for row in range(len(array)):
# 			for col in range(len(array[row])):
# 				if col == 1:
# 					xoptions = gtk.FILL|gtk.EXPAND
# 				else:
# 					xoptions = gtk.FILL
# 				table.attach(array[row][col],col,col+1,row,row+1,xoptions=xoptions,yoptions=0)
		
		vbox = gtk.VBox(spacing=8)
		vbox.pack_start(self._super_dynamic_mode)
		
		align = gtk.Alignment(xscale=1,yscale=1)
		align.set_padding(0,0,17,0)
		align.add(self._settings_table)
		vbox.pack_start(align)
		
		align = gtk.Alignment(xscale=1,yscale=1)
		align.set_padding(6,0,12,0)
		align.add(vbox)
		self.add(align)
		
		self._load_from_config()
		self.super_dynamic_mode_toggled(self._super_dynamic_mode) # manual callback to reflect state
		
		self._super_dynamic_mode.connect("toggled",self.super_dynamic_mode_toggled)
		self._apply_button.connect("clicked",self.apply_button_clicked)
		self._super_dynamic_mode.connect("toggled",self._config_changed)
		self._the_song_im_listening.connect("toggled",self._config_changed)
		self._the_song_im_listening_wander.connect("toggled",self._config_changed)
		self._the_last_song_in_playlist.connect("toggled",self._config_changed)
# 		self._after_the_song_im_listening.connect("toggled",self._config_changed)
		self._top_number_of_songs.connect("value_changed",self._config_changed)
		self._upcoming_songs.connect("value_changed",self._config_changed)
		self._weighting.connect("value_changed",self._config_changed)
# 		self._apply_button.set_no_show_all(True) # FIXME find a use for this button
		
	def _load_from_config(self):
		c = get_plugin_public_interface().get_config()["super_dynamic_mode"]
		self._super_dynamic_mode.set_active(c["enabled"])
		self._top_number_of_songs.set_value(c["top_choices"])
		self._upcoming_songs.set_value(c["upcoming_songs"])
# 		logger.debug("%s",c)
		if c["choice_mechanism"] == SONG_IM_LISTENING:
			self._the_song_im_listening.set_active(True)
		elif c["choice_mechanism"] == SONG_IM_LISTENING_WANDER:
			self._the_song_im_listening_wander.set_active(True)
		elif c["choice_mechanism"] == LAST_SONG_IN_PLAYLIST:
			self._the_last_song_in_playlist.set_active(True)
		else: assert(False)
		if c["insert_position"] == SONG_IM_LISTENING:
			self._after_the_song_im_listening.set_active(True)
		elif c["insert_position"] == LAST_SONG_IN_PLAYLIST:
			self._at_the_end_of_playlist.set_active(True)
		else: assert(False)
		tempo_weight,spectrum_weight = c["tempo_spectrum_weights"]
# 		assert(2 - tempo_weight == spectrum_weight)#sometimes 1 + 1 isn't 2 in the float universe
		weighting = 2 - tempo_weight
		weighting = spectrum_weight
		self._weighting.set_value(weighting)

		
	def get_active(self):
		return self._super_dynamic_mode.get_active()
	
	def get_number_of_top_choices(self):
		return int(self._top_number_of_songs.get_value())

	def get_number_of_upcoming_songs(self):
		return int(self._upcoming_songs.get_value())
	
	def based_on_the_song_im_listening(self):
		return self._the_song_im_listening.get_active()
	
	def based_on_the_song_im_listening_wander(self):
		return self._the_song_im_listening_wander.get_active()
	
	def based_on_the_last_song_in_playlist(self):
		return not self.based_on_the_song_im_listening()
	
	def insert_after_the_song_im_listening(self):
		return self._after_the_song_im_listening.get_active()
	
	def insert_at_the_end_of_playlist(self):
		return not self.insert_after_the_song_im_listening()
	
	def get_tempo_and_spectrum_weights(self):
		"""Return a tuple (tempo_weight,spectrum_weight)"""

		weighting = self._weighting.get_value()
		
# 		si slider hacia tempo, value = 0
# 		si slider hacia spectrum value = 2
# 		si slider en la mitad value = 1
		
		tempo_weight = 2 - weighting
		spectrum_weight = weighting
		return (tempo_weight,spectrum_weight)

	def super_dynamic_mode_toggled(self,widget,e=None):
		self._settings_table.set_sensitive(widget.get_active())
		self.apply_button_clicked(self._apply_button) # to make preferences be saved
		
	def apply_button_clicked(self,widget,e=None):
		logger.debug("Apply button clicked")
		
	def _config_changed(self,widget,e=None):
# 		logger.debug("Configuration changed")
		try:
			c = get_plugin_public_interface().get_config()["super_dynamic_mode"]
			c["enabled"] = self.get_active()
			c["top_choices"] = self.get_number_of_top_choices()
			c["upcoming_songs"] = self.get_number_of_upcoming_songs()
			if self.based_on_the_song_im_listening():
				c["choice_mechanism"] = SONG_IM_LISTENING
			elif self.based_on_the_song_im_listening_wander():
				c["choice_mechanism"] = SONG_IM_LISTENING_WANDER
			elif self.based_on_the_last_song_in_playlist():
				c["choice_mechanism"] = LAST_SONG_IN_PLAYLIST
			else: assert(False)
			if self.insert_after_the_song_im_listening():
				c["insert_position"] = SONG_IM_LISTENING
			elif self.insert_at_the_end_of_playlist():
				c["insert_position"] = LAST_SONG_IN_PLAYLIST
			else: assert(False)
			c["tempo_spectrum_weights"] = self.get_tempo_and_spectrum_weights()
			
# 			logger.debug("Configuration object data:")
# 			for key,val in c.iteritems():
# 				logger.debug("%s: %s",key,val)
				
			get_plugin_public_interface().notify_config_changed()
		except AmarokDcopFailed,e:
			if e.output == "output call failed":
				logger.debug("amaroK is unexpectedly gone")
			else: raise
		except:
			logger.exception("Something failed")

		
class ConfigWindowProgressFrame(gtk.Frame):
	def __init__(self):
		gtk.Frame.__init__(self,"")
		self.set_shadow_type(gtk.SHADOW_NONE)
		self.get_label_widget().set_markup("<b>Analysis status</b>")
		align = gtk.Alignment(xscale=1,yscale=1)
		align.set_padding(6,0,12,0)
		self.vbox = gtk.VBox(spacing=12)
		self.song_label = gtk.Label("Gathering background process information...")
		import pango
		self.song_label.set_property("ellipsize",pango.ELLIPSIZE_MIDDLE)
		self.song_label.set_alignment(0,0.5)
		self.pqueue_label = gtk.Label("Gathering priority queue information...")
		self.pqueue_label.set_property("ellipsize",pango.ELLIPSIZE_MIDDLE)
		self.pqueue_label.set_alignment(0,0.5)
		self.progressbar = gtk.ProgressBar()
		self.progressbar.set_text("Wait...")
		align.add(self.vbox)
		self.add(align)
		self.vbox.pack_start(self.song_label,expand=False,fill=False)
		self.vbox.pack_start(self.progressbar,expand=True,fill=True)
		self.vbox.pack_start(self.pqueue_label,expand=True,fill=True)
		
	

# class AboutWindow(gtk.Window):
# 		
# 	def __init__(self):
# 		gtk.Window.__init__(self)
# 		logger.debug("About window")
# 		self.set_title("About Smart DJ")
# 		self.set_resizable(False)
# 		self.set_border_width(12)
# 		vbox = gtk.VBox(spacing=12)
# 		l = gtk.Label("")
# 		l.set_markup("<big><big><b>Smart DJ</b></big></big>")
# 		vbox.pack_start(l)
# 		
# 		
# 		def get_version():
# 			import sys
# 			script = sys.argv[0]
# 			import os
# 			dirname = os.path.dirname(script)
# 			vfile = os.path.join(dirname,"version")
# 			version = file(vfile,"r").read(-1).strip()
# 			return version
# 		
# 		l = gtk.Label("")
# 		version = get_version()
# 		l.set_markup("<b>version %s</b>"%version)
# 		vbox.pack_start(l)
# 		
# 		l = gtk.Label("")
# 		author = "Manuel Amador (Rudd-O) &lt;rudd-o@amautacorp.com&gt;"
# 		l.set_markup("<big><i>written by %s</i></big>"%author)
# 		vbox.pack_start(l)
# 		
# 		self.closebutton = gtk.Button(stock=gtk.STOCK_CLOSE)
# 		self.closebutton.connect_object("clicked",gtk.Window.destroy,self)
# 		align = gtk.HButtonBox()
# 		align.set_spacing(12)
# 		align.set_layout(gtk.BUTTONBOX_END)
# 		align.add(self.closebutton)
# 		vbox.pack_start(align,expand=True,fill=True)
# 		self.add(vbox)
# 
class ConfigWindow(gtk.Window):
	
	
# 	def show_about(self,w=None,e=None):
# 		f = AboutWindow()
# 		f.set_transient_for(self)
# 		f.show_all()
# 		
	def __init__(self):
		gtk.Window.__init__(self)
		logger.debug("Config window")
		self.set_title("Smart DJ preferences")
		self.set_resizable(False)
# 		self.set_size_request(500,-1)
		self.set_border_width(12)
		vbox = gtk.VBox(spacing=12)
		self.progress_frame = ConfigWindowProgressFrame()
		self.dynamic_mode_frame = ConfigWindowDynamicModeFrame()
		self.closebutton = gtk.Button(stock=gtk.STOCK_CLOSE)
# 		self.aboutbutton = gtk.Button(stock=gtk.STOCK_ABOUT)
		self.closebutton.connect_object("clicked",gtk.Window.hide,self)
# 		self.aboutbutton.connect("clicked",self.show_about)
		self.connect_object("delete_event",ConfigWindow.on_delete_event,self)
		vbox.pack_start(self.dynamic_mode_frame,expand=False,fill=False)
		vbox.pack_start(self.progress_frame,expand=False,fill=False)
# 		ruler = gtk.HSeparator()
# 		vbox.pack_start(ruler,expand=False,fill=False)
		align = gtk.HButtonBox()
		align.set_spacing(12)
		align.set_layout(gtk.BUTTONBOX_END)
# 		align.add(self.aboutbutton)
# 		align.set_child_secondary(self.aboutbutton,True)
		align.add(self.closebutton)
		vbox.pack_start(align,expand=True,fill=True)
		self.add(vbox)
		vbox.show_all()
		
		gobject.timeout_add(5000,self.update)
		
	def show(self):
		logger.info("Showing configuration window")
		gtk.Window.show(self)
		self.update()
		
	def present(self):
		logger.info("Presenting configuration window")
		gtk.Window.present(self)
		self.update()
		
	def on_delete_event(self,w,e=None):
		self.hide()
		return True
		
	def update(self,persistent=True):
		if not self.get_property("visible"): return persistent
		def _update_t():
# 			raise Exception,"YEAH YEAH"
			import time
			a = time.time()
			current_song = get_plugin_public_interface().get_current_song()
			pqueue_count = get_plugin_public_interface().get_priority_queue_count()
			total = get_plugin_public_interface().get_total_songs()
			left = get_plugin_public_interface().get_remaining_songs()
			gobject.idle_add(self._real_update,current_song,pqueue_count,total,left)
			logger.info("Updating configuration window took %s"%(time.time() -a))
			
		t = Thread(target=_update_t)
		t.setDaemon(True)
		t.start()
		return persistent
		
	def _real_update(self,current_song,pqueue_count,total,left):
		
		progress = ( float(total) - float(left) ) / float(total)
		if current_song:
			self.progress_frame.song_label.set_markup("Analyzing %s"%current_song)
			string = "%s of %s songs (%s%%) pending analysis"%(left,total,left*100/total)
		else:
			self.progress_frame.song_label.set_markup("Idle.  All songs have been analyzed.")
			string = "%s songs analyzed"%(total)
		if pqueue_count:
			if pqueue_count != 1: s = "s"
			else: s = ""
			self.progress_frame.pqueue_label.set_markup("%s song%s in the analysis priority queue"%(pqueue_count,s))
		else:
			self.progress_frame.pqueue_label.set_markup("No songs in the analysis priority queue")
		
		self.progress_frame.progressbar.set_fraction(progress)
		self.progress_frame.progressbar.set_text(string)
	

def _show_config_sync():
	try:
		t=_show_config_sync
		if not hasattr(t,"window"): t.window = ConfigWindow()
		t.window.present()
		
	except AmarokDcopFailed,e:
		if e.output == "output call failed":
			logger.debug("amaroK is unexpectedly gone")
		else: raise
	except:
		logger.exception("Something went wrong in _show_config_sync")
		HIGErrorAlert(None,"Error while showing configuration window","Cannot show configuration window","An unknown error just occurred.  Please report the contents of the file %s to the Smart DJ plug-in developers."%os.path.expanduser(n_logfile)).show()


def _run_ui_loop():
	try:
		gtk.gdk.threads_init()
		gtk.gdk.threads_enter()
		gtk.main()
	except:
		logger.exception("Something went wrong in the UI loop thread")

#end private implementation


# public API

def initialize(public_interface):
	logger.debug("Initializing with public interface %s",public_interface)
	_set_plugin_public_interface(public_interface)
	t = Thread(target=_run_ui_loop)
	t.setDaemon(True)
	t.start()

def search(start_files):
	""" Receive a Python list of file paths (NOT URLs)"""
	gobject.idle_add(_search_sync,start_files)

def show_configuration():
	""" Receive a Python list of file paths (NOT URLs)"""
	gobject.idle_add(_show_config_sync)
